﻿// Copyright (c) 2024, itsdave GmbH and contributors
// For license information, please see license.txt

function loadLeafletAndInitMap(frm, callback) {
    if (typeof callback === "function") callback(frm); // Leaftlet is already loaded, just run the callback
}

function create_dialog_and_init_map(frm) {
    const dialog = new frappe.ui.Dialog({
        title: 'Show Route',
        fields: [{ fieldtype: 'HTML', fieldname: 'map_html', label: 'Map', options: '<div id="map" style="height:calc(80vh - 20px);"></div>' }],
        primary_action_label: 'Close',
        primary_action() {
            if (currentMap) {
                currentMap.remove();  // Remove the map when closing the dialog
                currentMap = null;  // unset the reference
            }
            dialog.hide();
        },
        size: 'extra-large'
    });

    // Adjust the size of the dialog dynamically to 80% of the browser window
    const modalDialog = dialog.$wrapper.find('.modal-content');
    modalDialog.css({
	"width": "85vw",
        "max-width": "85vw",
	"margin-left": "-25vh"
    });

    // Add hide event to reload the page when the dialog is closed
    dialog.onhide = function () {
        window.location.reload();
    };

    dialog.show();
    setTimeout(() => init_map(frm), 500); // Add a delay to ensure the map loads correctly
}

var currentMap = null;

function init_map(frm) {
    console.table(window.L);
    console.log("Initializing map with MyLeaflet...");
    if (currentMap !== null) {
        currentMap.remove();  // Remove the previous map if it exists
    }
    var map = window.L.map('map').setView([52.998919, 10.0654483], 13);
    window.L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
        maxZoom: 19,
        attribution: '© OpenStreetMap'
    }).addTo(map);
    currentMap = map;

    // Create a layer group for checkpoint markers
    var checkpointMarkers = window.L.layerGroup();

    // Control to toggle checkpoint markers
    var control = window.L.control({ position: 'topleft' });
    control.onAdd = function (map) {
        var div = window.L.DomUtil.create('div', 'leaflet-control leaflet-bar');
        var button = window.L.DomUtil.create('a', '', div);
        button.innerHTML = '📍';
        button.href = '#';

        window.L.DomEvent.on(button, 'click', function (e) {
            window.L.DomEvent.stop(e);

            if (map.hasLayer(checkpointMarkers)) {
                map.removeLayer(checkpointMarkers); // turn off
            } else {
                map.addLayer(checkpointMarkers); // turn on
            }
        });

        return div;
    };
    control.addTo(map);

    // call data from api
    frappe.call({
        method: "fleet_management.traccar_api.getroutes",
        args: {
            "name": frm.name,
            "start": frm.start,
            "end": frm.end
        },
        callback: function (r) {
            if (r.message) {
                console.table(r.message);
                const data = r.message;
                const routeCoordinates = data.map(entry => [entry.latitude, entry.longitude]);
                const polyline = window.L.polyline(routeCoordinates, { color: 'red' }).addTo(map);
                map.fitBounds(polyline.getBounds());

                // Add differentiated markers
                data.forEach((point, index) => {
                    let markerColor, layerToAdd = map;
                    let popupMessage = `Time: ${moment(point.deviceTime).tz("Europe/Berlin").format('DD/MM/YYYY HH:mm:ss')}<br>Speed: ${parseFloat(point.speed).toFixed(2)} km/h`;

                    if (index === 0) { // Point of start
                        markerColor = 'green';
                    } else if (index === data.length - 1) { // Point of end
                        markerColor = 'blue';
                    } else { // Points in between
                        markerColor = 'yellow';
                        layerToAdd = checkpointMarkers; // Add these markers to the checkpoint layer
                    }

                    const icon = new window.L.Icon({
                        iconUrl: `https://raw.githubusercontent.com/pointhi/leaflet-color-markers/master/img/marker-icon-2x-${markerColor}.png`,
                        shadowUrl: 'https://cdnjs.cloudflare.com/ajax/libs/leaflet/0.7.7/images/marker-shadow.png',
                        iconSize: [20, 32],
                        iconAnchor: [10, 32],
                        popupAnchor: [1, -32],
                        shadowSize: [32, 32]
                    });

                    window.L.marker([point.latitude, point.longitude], { icon: icon }).addTo(layerToAdd)
                        .bindPopup(popupMessage);
                });
            }
        },
        error: function (error) {
            console.error('Error loading the route data:', error);
            frappe.msgprint('Error loading the route data');
        }
    });
}

function show_map(frm) {
    console.log("Loading map...");
    loadLeafletAndInitMap(frm, create_dialog_and_init_map);
}

frappe.listview_settings['Trip'] = {
    button: {
        show: function (doc) {
            return true;
        },
        get_label: function () {
            return __('🗺️ Map route');
        },
        get_description: function (doc) {
            return __('Show map route');
        },
        action: function (doc) {
            show_map(doc);
        }
    },
    formatters: {
        average_speed(val) {
            return (parseFloat(val) * 1.852).toFixed(2) + " km/h"; // Convert knots to km/h
            //return parseFloat(val) + " km/h"; // Convert knots to km/h
        },
        distance(val) {
            return (parseFloat(val) / 1000).toFixed(2) + " km"; // Convert meters to km
            //return parseFloat(val) + " km"; // Convert meters to km
        }
    },
    onload(listview) {
        listview.page.add_inner_button('Force Route Synchronization', function () {
            frappe.show_alert({ indicator: 'blue', message: __("🔄 Synchronizing data...") });
            frappe.call({
                method: 'fleet_management.tasks.get_trips_for_device',
                freeze: true,  // Option to freeze the user interface with a transparent overlay
                freeze_message: __("🔄 Synchronizing data..."), // Message to show when the user interface is frozen
                callback: function (r) {
                    if (!r.exc) {
                        frappe.hide_msgprint(); // Hide the synchronization message / spinner
                        location.reload(); // Reload the page after completion
                    } else {
                        frappe.show_alert({ message: __("Synchronization failed"), indicator: 'red' });
                    }
                }
            });
        });
    }
};
